import datetime

from bs4 import BeautifulSoup
from urllib.parse import unquote

from Attrap import Attrap


class Attrap_prefdpt(Attrap):

    user_agent = 'Mozilla/5.0 (Windows NT 10.0; rv:109.0) Gecko/20100101 Firefox/115.0'

    # Paramètres par défaut des cartes grises et blanches. Devrait la plupart du temps être surchargés par la classe de préfecture de département
    grey_card = {
        'regex': {
            'year': None,
            'month': None
        },
        'css_path': {
            'title': 'div.fr-card.fr-card--sm.fr-card--grey.fr-enlarge-link div.fr-card__body div.fr-card__content h2.fr-card__title a'
        },
        'link_to_raa': False,
        'follow_link_on_unrecognised_date': True
    }
    white_card = {
        'regex': {
            'year': None,
            'month': None,
        },
        'css_path': {
            'title': 'div.fr-card.fr-card--horizontal.fr-card--sm.fr-enlarge-link div.fr-card__body div.fr-card__content h2.fr-card__title a.fr-card__link',  # Chemin CSS du titre des cartes blanches
            'details': 'div.fr-card.fr-card--horizontal.fr-card--sm.fr-enlarge-link div.fr-card__body div.fr-card__content div.fr-card__end p.fr-card__detail',  # Chemin CSS du détail des cartes blanches
            'pager': 'ul.fr-pagination__list li a.fr-pagination__link.fr-pagination__link--next.fr-pagination__link--lg-label'  # Chemin CSS du pager des cartes blanches
        },
        'link_to_raa': False,
        'follow_link_on_unrecognised_date': True
    }

    # Liste des widgets à analyser (sera remplie au moment de l'initialisation, mais peut être surchargée par la classe de préfecture de département)
    widgets = []

    # Est-ce qu'on inclue les widgets des cartes blanches et grises ? Par défaut oui, mais il peut être nécessaire de les désactiver sur certaines préfectures
    include_grey_card_widget = True
    include_white_card_widget = True

    # Chemin CSS vers un RAA
    element_css_path = 'div.fr-downloads-group.fr-downloads-group--bordered ul li a,div a.fr-link.fr-link--download'

    # Temporisation (en secondes) entre chaque requête (ne devrait pas avoir à être changée)
    pref_sleep_time = 5

    class DptWidget:
        """Une classe représentant un widget sur le site d'une préfecture de département."""

        def __init__(self, name, regex=None, css_path=None, link_to_raa=False, follow_link_on_unrecognised_date=True):
            self.name = name
            self.regex = regex
            self.css_path = css_path
            self.link_to_raa = link_to_raa
            self.follow_link_on_unrecognised_date = follow_link_on_unrecognised_date

        def has_css_path(self, key):
            return self.css_path and self.css_path.get(key, None) is not None

        def get_css_path(self, key):
            if not self.has_css_path(key):
                return None
            else:
                return self.css_path.get(key, None)

        def has_regex(self, key):
            return self.regex and self.regex.get(key, None) is not None

        def get_regex(self, key):
            if not self.has_regex(key):
                return None
            else:
                return self.regex.get(key, None)

    def __init__(self, data_dir):
        """Une classe générique permettant d'analyser les préfectures de département en fonction de certains paramètres."""

        super().__init__(data_dir, self.user_agent)
        self.set_sleep_time(self.pref_sleep_time)

        if isinstance(self.raa_page, str):
            self.raa_page = [self.raa_page]
        self.page_urls_to_parse = self.raa_page
        self.elements = []

        # On ajoute les cartes grises et blanches à la liste des widgets à parser
        if self.include_grey_card_widget:
            self.widgets.append(
                Attrap_prefdpt.DptWidget(
                    'grey_card',
                    regex=self.grey_card['regex'],
                    css_path=self.grey_card['css_path'],
                    link_to_raa=self.grey_card['link_to_raa'],
                    follow_link_on_unrecognised_date=self.grey_card['follow_link_on_unrecognised_date']
                )
            )

        if self.include_white_card_widget:
            self.widgets.append(
                Attrap_prefdpt.DptWidget(
                    'white_card',
                    regex=self.white_card['regex'],
                    css_path=self.white_card['css_path'],
                    link_to_raa=self.white_card['link_to_raa'],
                    follow_link_on_unrecognised_date=self.white_card['follow_link_on_unrecognised_date']
                )
            )

    def get_raa(self, keywords):
        while not self.page_urls_to_parse == []:
            page_url = self.page_urls_to_parse[-1]
            page_content = self.get_page(page_url, 'get').content  # On récupère le HTML de la page
            self.parse_widgets(page_content)  # On parse les cartes blanches
            for element in self.get_raa_elements(page_content):  # On cherche les RAA
                self.elements.append(element)
            self.page_urls_to_parse.remove(page_url)  # On supprime la page de la liste de celles à parser

        self.parse_raa(self.elements[::-1], keywords)
        self.mailer()

    def parse_widgets(self, page_content):
        # Pour chaque widget paramétré, on le cherche sur la page
        for widget in self.widgets:
            cards = []
            # Si n'appelle pas la même fonction le widget a prévu un pager ou non
            if widget.has_css_path('pager'):
                cards = self.get_sub_pages_with_pager(
                    page_content,
                    widget.get_css_path('title'),  # Titre du lien
                    widget.get_css_path('pager'),  # Pager
                    widget.get_css_path('details'),  # Détails
                    self.hostname
                )
            else:
                cards = self.get_sub_pages(
                    page_content,
                    widget.get_css_path('title'),
                    self.hostname,
                    False
                )
            for card in cards:
                if card['url'] not in self.page_urls_to_parse:
                    date = None
                    date_is_correct = False

                    # Si un regex d'année est spécifié, on parse le titre avec
                    if widget.has_regex('year'):
                        date = Attrap.guess_date(card['name'].strip(), widget.get_regex('year')).replace(day=1, month=1)
                        # Si une date a été trouvée (l'année n'est pas 9999) et qu'elle est avant la valeur not_before, on la marque comme correcte
                        if date >= self.not_before.replace(day=1, month=1) and date.year < 9999:
                            date_is_correct = True

                    # Si un regex de mois est spécifié et qu'aucune date correcte n'a été trouvée, on teste avec le regex de mois sur le titre
                    if widget.has_regex('month') and not date_is_correct:
                        date = Attrap.guess_date(card['name'].strip(), widget.get_regex('month')).replace(day=1)
                        if date >= self.not_before.replace(day=1) and date.year < 9999:
                            date_is_correct = True

                    # Si un chemin CSS vers les détails du widget est spécifié et qu'aucune date correcte n'a été trouvée, on tente de parser la date présente dans les détails
                    if widget.has_css_path('details') and not date_is_correct:
                        try:
                            date = datetime.datetime.strptime(card['details'].replace('Publié le ', '').strip(), '%d/%m/%Y')
                            if date >= self.not_before:
                                date_is_correct = True
                        except Exception as e:
                            date = datetime.datetime(9999, 1, 1)

                    # Si la configuration indique que les liens renvoient vers un RAA, on ajoute le lien à la liste des éléments
                    if widget.link_to_raa:
                        if date.year == 9999:
                            date = None
                        raa = Attrap.RAA(card['url'], date, card['name'].strip(), timezone=self.timezone)
                        self.elements.append(raa)
                    else:
                        # Si une date a été trouvée, on regarde s'il faut ajouter l'URL à la liste des pages à parser
                        if date_is_correct or (date.year == 9999 and widget.follow_link_on_unrecognised_date):
                            self.page_urls_to_parse.append(card['url'])

    def get_raa_elements(self, page_content):
        elements = []
        # On charge le parser
        soup = BeautifulSoup(page_content, 'html.parser')

        # On récupère chaque balise a
        for a in soup.select(self.element_css_path):
            if a.get('href'):
                if a['href'].startswith('/'):
                    url = f"{self.hostname}{a['href']}"
                else:
                    url = a['href']

                url = unquote(url)
                name = a.find('span').previous_sibling.replace('Télécharger ', '').strip()
                date = datetime.datetime.strptime(a.find('span').get_text().split(' - ')[-1].strip(), '%d/%m/%Y')

                raa = Attrap.RAA(url, date, name, timezone=self.timezone)
                elements.append(raa)
        return elements
